import math
import copy
from .delaunay import *
from .hull import *

class PolarPoint:
    def __init__(self, r, theta):
        '''
            Creates  a polar point.
            :param int r: The radial distance.
            :param int theta: Polar angle.
            :return: Object of Polar Point
            :rtype: Object
            :raises ValueError: if the theta exceeds abs(360).
            :raises TypeError: if the message_body is not a basestring
        '''
        self.r = r
        if abs(theta) > 360 :
            raise ValueError
        elif theta < 0 :
            self.theta = 360 - theta
        else :
            self.theta = theta
    def toCratesian(self):
        return Point(self.r*math.cos(self.theta),self.r*math.sin(self.theta))

class Point:
    def __init__(self, x, y):
        self.x = x
        self.y = y
    def __str__(self):
        return '('+str(self.x)+' , ' + str(self.y)+')'
    @staticmethod
    def orientation(p, q, r):
        val = (float(q.y - p.y) * float(r.x - q.x)) - (float(q.x - p.x) * float(r.y - q.y))
        if (val > 0):
            return 1
        elif (val < 0):
            return 2
        else:
            return 0
    @staticmethod
    def findQuadrant(p):
        if p.x==0 and p.y==0 :
            return "0"
        if p.x>0 and p.y>0 :
            return "1"
        if p.x<0 and p.y>0 :
            return "2"
        if p.x<0 and p.y<0 :
            return "3"
        if p.x>0 and p.y<0 :
            return "4"
        if p.y<0 and p.x==0 :
            return "-y"
        if p.y>0 and p.x==0 :
            return "+y"
        if p.x>0 and p.y==0 :
            return "+x"
        if p.x<0 and p.y==0 :
            return "-x"
    @classmethod 
    def euclideanDistance(cls, p, q):
        '''To find Euclidean Distance of two Points.'''
        return math.sqrt((p.x-q.x)*(p.x-q.x) + (p.y-q.y)*(p.y-q.y))
    
    @classmethod 
    def manhattanDistance(cls, p, q):
        """ # To find Manhattan Distance of two Points. """
        return abs(p.x-q.x) + abs(p.y-q.y)
    @classmethod 
    def centroid(cls,listOfPoints):
        """ #Cnetroid of a list of Points """
        noOfPoints = len(listOfPoints)
        if noOfPoints==0:
            raise ValueError
        sigmaX = 0
        sigmaY = 0 
        for point in listOfPoints:
            sigmaX += point.x
            sigmaY += point.y
        centroidPoint = Point(sigmaX/noOfPoints, sigmaY/noOfPoints)
        return centroidPoint
    @classmethod
    def convexHull(cls, listOfPoints):
        return CONVEXHULL(listOfPoints)
    # No of lattice points #No of Lattice Points inside a Polygon by Pick's Theorem
    @classmethod
    def pointsOnLine(cls,p, q):
        return math.gcd(q.x - p.x, q.y - p.y) + 1
    @classmethod
    def pointsInsidePolygon(cls, listOfPoints):
        A = int(Point.area(listOfPoints))
        B = 0
        listOfPoints.append(listOfPoints[0]);
        for i in range(0,len(listOfPoints)):
            B += Point.pointsOnLine(listOfPoints[i], listOfPoints[i+1])
        return A + B/2 - 1

# ======================================================================
    # Find the closest pair of points using Divide And Conqure
    @classmethod
    def closestPairOfPoints(cls,listOfPoints):
        n = len(listOfPoints)
        listOfPoints.sort(key = lambda point: point.x)
        Q = copy.deepcopy(listOfPoints)
        Q.sort(key = lambda point: point.y)    
        return Point.closestUtil(listOfPoints, Q, n)
    @staticmethod
    def stripClosest(strip, size, d):
        min_val = d 
        for i in range(size):
            j = i + 1
            while j < size and (strip[j].y - strip[i].y) < min_val:
                min_val = Point.euclideanDistance(strip[i], strip[j])
                j += 1
        return min_val
    @staticmethod
    def closestUtil(P, Q, n):
        mid = n // 2
        midPoint = P[mid]
        dl = Point.closestUtil(P[:mid], Q, mid)
        dr = Point.closestUtil(P[mid:], Q, n - mid) 
        d = min(dl, dr)
        strip = [] 
        for i in range(n): 
            if abs(Q[i].x - midPoint.x) < d: strip.append(Q[i])
        return min(d, Point.stripClosest(strip, len(strip), d))

# ==================================================================
    @classmethod
    def delaunayTriangulation(cls, listOfPoints, listOfList=False):
        points = []
        for Point in listOfPoints:
            points.append([Point.x,Point.y])
        triangles = Delaunator(points).triangles
        if listOfList == True:
            coordinates = []
            for i in range(0, len(triangles), 3):
                coordinates.append([points[triangles[i]], points[triangles[i + 1]], points[triangles[i + 2]]])
                return coordinates
        return triangles

# ===========================TEST============================================
# p1 = Point(1,1)
# print(p1)
# pp1 = PolarPoint(2,30)
# p2 = pp1.toCratesian()
# print(p2)
# p3 = Point(2,2)
# print(Point.orientation(p1,p2,p3))
# print(Point.findQuadrant(p2))
# print(Point.euclideanDistance(p1,p2))
# print(Point.manhattanDistance(p1,p2))
# l = [p1,p2,p3]
# print(Point.centroid(l))
# print(Point.convexHull(l))
# # print(Point.closestPairOfPoints(l))
# print(Point.delaunayTriangulation(l))


import math

EPSILON = math.pow(2,-52)
EDGE_STACK =[None] * 512

class Delaunator:
    """ Fast Delaunay triangulation of 2D points implemented in Python. 
        from Delaunator import Delaunator
        points = [[168, 180], [168, 178], [168, 179], [168, 181], [168, 183], ...]
        triangles = Delaunator(points).triangles
        print(triangles)

        >> [623, 636, 619,  636, 444, 619, ...]

        for i in range(0, len(triangles), 3):
            coordinates.append([points[triangles[i]], points[triangles[i + 1]], points[triangles[i + 2]]])
    
    """
    def __init__(self,points):
        n = len(points)

        if (len(points) < 3):
            raise ValueError("Need at least 3 points")
        coords = [None] * n * 2

        for i in range(0,n):
            p = points[i]
            coords[2 * i] = (p[0])
            coords[2 * i+1] = (p[1])
        triangles = self.constructor(coords)

    def constructor(self, coords):
        n = len(coords) >> 1

        self.coords = coords

        # arrays that will store the triangulation graph
        maxTriangles = max(2 * n - 5, 0)
        self._triangles = [None] * maxTriangles * 3
        self._halfedges = [None] * maxTriangles * 3

        # temporary arrays for tracking the edges of the advancing convex hull
        self.hashSize = math.ceil(math.sqrt(n))
        self.hullPrev = [None] * n # edge to prev edge
        self.hullNext = [None] * n # edge to next edge
        self.hullTri = [None] * n # edge to adjacent triangle
        self.hullHash = [-1] * self.hashSize # angular edge hash

        # temporary arrays for sorting points
        self._ids =  [None] * n
        self._dists = [None] * n
        triangles = self.update(coords)

        return triangles

    def update(self,coords):
        n = len(coords) >> 1

        # populate an array of point indices; calculate input data bbox
        minX = math.inf
        minY = math.inf
        maxX = -math.inf
        maxY = -math.inf

        for i in range(0,n):
            x = coords[2 * i]
            y = coords[2 * i + 1]
            if (x < minX): minX = x
            if (y < minY): minY = y
            if (x > maxX): maxX = x
            if (y > maxY): maxY = y
            self._ids[i] = i

        cx = (minX + maxX) / 2
        cy = (minY + maxY) / 2

        minDist = math.inf
        i0 = 0
        i1 = 0
        i2 = 0

        # pick a seed point close to the center
        for i in range(0,n):
            d = dist(cx, cy, coords[2 * i], coords[2 * i + 1])

            if (d < minDist):
                i0 = i
                minDist = d

        i0x = coords[2 * i0]
        i0y = coords[2 * i0 + 1]
        minDist = math.inf

        # find the point closest to the seed
        for i in range(0,n):
            if (i == i0): continue
            d = dist(i0x, i0y, coords[2 * i], coords[2 * i + 1])

            if (d < minDist and d > 0):
                i1 = i
                minDist = d

        i1x = coords[2 * i1]
        i1y = coords[2 * i1 + 1]

        minRadius = math.inf

        # find the third point which forms the smallest circumcircle with the first two
        for i in range(0,n):
            if (i == i0 or i == i1): continue
            r = circumradius(i0x, i0y, i1x, i1y, coords[2 * i], coords[2 * i + 1])

            if (r < minRadius):
                i2 = i
                minRadius = r

        i2x = coords[2 * i2]
        i2y = coords[2 * i2 + 1]

        if (minRadius == math.inf):
            # order collinear points by dx (or dy if all x are identical)
            # and return the list as a hull
            for i in range(0,n):
                self._dists[i] = (coords[2 * i] - coords[0]) or (coords[2 * i + 1] - coords[1])

            quicksort(self._ids, self._dists, 0, n - 1)
            hull =  [None] * n
            j = 0
            d0 = -math.inf

            for i in range(0,n):
                id = self._ids[i]

                if (self._dists[id] > d0):
                    hull[j] = id
                    j+=1
                    d0 = self._dists[id]

            self.hull = hull[0:j]
            self.triangles =  []
            self.halfedges =  []

        # swap the order of the seed points for counter-clockwise orientation
        if (orient(i0x, i0y, i1x, i1y, i2x, i2y)):
            i = i1
            x = i1x
            y = i1y
            i1 = i2
            i1x = i2x
            i1y = i2y
            i2 = i
            i2x = x
            i2y = y

        center = circumcenter(i0x, i0y, i1x, i1y, i2x, i2y)
        self._cx = center[0]
        self._cy = center[1]

        for i in range(0,n):
            self._dists[i] = dist(coords[2 * i], coords[2 * i + 1], center[0], center[1])

        # sort the points by distance from the seed triangle circumcenter
        quicksort(self._ids, self._dists, 0, n - 1)

        # set up the seed triangle as the starting hull
        self._hullStart = i0
        hullSize = 3

        self.hullNext[i0] = self.hullPrev[i2] = i1
        self.hullNext[i1] = self.hullPrev[i0] = i2
        self.hullNext[i2] = self.hullPrev[i1] = i0

        self.hullTri[i0] = 0
        self.hullTri[i1] = 1
        self.hullTri[i2] = 2

        self.hullHash[self._hashKey(i0x, i0y)] = i0
        self.hullHash[self._hashKey(i1x, i1y)] = i1
        self.hullHash[self._hashKey(i2x, i2y)] = i2

        self.trianglesLen = 0
        self._addTriangle(i0, i1, i2, -1, -1, -1)

        xp=0
        yp=0

        for k in range(0,len(self._ids)):
            i = self._ids[k]
            x = coords[2 * i]
            y = coords[2 * i + 1]

            # skip near-duplicate points
            if (k > 0 and abs(x - xp) <= EPSILON and abs(y - yp) <= EPSILON): continue

            xp = x
            yp = y

            # skip seed triangle points
            if (i == i0 or i == i1 or i == i2): continue

            # find a visible edge on the convex hull using edge hash
            start = 0
            key = self._hashKey(x, y)

            for j in range(0,self.hashSize):
                start = self.hullHash[(key + j) % self.hashSize]
                if (start != -1 and start != self.hullNext[start]): break

            start = self.hullPrev[start]
            e = start

            while True:
                q = self.hullNext[e]
                if orient(x, y, coords[2 * e], coords[2 * e + 1], coords[2 * q], coords[2 * q + 1]): break
                e = q

                if (e == start):
                    e = -1
                    break

            if (e == -1): continue # likely a near-duplicate point; skip it

            # add the first triangle from the point
            t = self._addTriangle(e, i, self.hullNext[e], -1, -1, self.hullTri[e])

            # recursively flip triangles from the point until they satisfy the Delaunay condition
            self.hullTri[i] = self._legalize(t + 2,coords)
            self.hullTri[e] = t # keep track of boundary triangles on the hull
            hullSize+=1

            # walk forward through the hull, adding more triangles and flipping recursively
            n = self.hullNext[e]

            while True:
                q = self.hullNext[n]
                if not (orient(x, y, coords[2 * n], coords[2 * n + 1], coords[2 * q], coords[2 * q + 1])): break
                t = self._addTriangle(n, i, q, self.hullTri[i], -1, self.hullTri[n])
                self.hullTri[i] = self._legalize(t + 2,coords)
                self.hullNext[n] = n # mark as removed
                hullSize-=1
                n = q

            # walk backward from the other side, adding more triangles and flipping
            if (e == start):
                while True:
                    q = self.hullPrev[e]
                    if not (orient(x, y, coords[2 * q], coords[2 * q + 1], coords[2 * e], coords[2 * e + 1])): break
                    t = self._addTriangle(q, i, e, -1, self.hullTri[e], self.hullTri[q])
                    self._legalize(t + 2,coords)
                    self.hullTri[q] = t
                    self.hullNext[e] = e # mark as removed
                    hullSize-=1
                    e = q

            # update the hull indices
            self._hullStart = self.hullPrev[i] = e
            self.hullNext[e] = self.hullPrev[n] = i
            self.hullNext[i] = n

            # save the two new edges in the hash table
            self.hullHash[self._hashKey(x, y)] = i
            self.hullHash[self._hashKey(coords[2 * e], coords[2 * e + 1])] = e

        self.hull = [None] * hullSize
        e = self._hullStart
        for i in range(0,hullSize):
            self.hull[i] = e
            e = self.hullNext[e]

        # trim typed triangle mesh arrays
        self.triangles = self._triangles[0:self.trianglesLen]
        self.halfedges = self._halfedges[0:self.trianglesLen]

        return self.triangles

    def _hashKey(self,x, y):
        return math.floor(pseudoAngle(x - self._cx, y - self._cy) * self.hashSize) % self.hashSize

    def _legalize(self,a,coords):
        i = 0
        ar = 0

        # recursion eliminated with a fixed-size stack
        while True:
            b = self._halfedges[a]
            """
              if the pair of triangles doesn't satisfy the Delaunay condition
              (p1 is inside the circumcircle of [p0, pl, pr]), flip them,
              then do the same check/flip recursively for the new pair of triangles
             
                        pl                    pl
                       /||\                  /  \
                    al/ || \bl            al/    \a
                     /  ||  \              /      \
                    /  a||b  \    flip    /___ar___\
                  p0\   ||   /p1   =>   p0\---bl---/p1
                     \  ||  /              \      /
                    ar\ || /br             b\    /br
                       \||/                  \  /
                        pr                    pr
             
            """
            a0 = a - a % 3
            ar = a0 + (a + 2) % 3

            if (b == -1): # convex hull edge
                if (i == 0): break
                i-=1
                a = EDGE_STACK[i]
                continue

            b0 = b - b % 3
            al = a0 + (a + 1) % 3
            bl = b0 + (b + 2) % 3

            p0 = self._triangles[ar]
            pr = self._triangles[a]
            pl = self._triangles[al]
            p1 = self._triangles[bl]

            illegal = inCircle(
                coords[2 * p0], coords[2 * p0 + 1],
                coords[2 * pr], coords[2 * pr + 1],
                coords[2 * pl], coords[2 * pl + 1],
                coords[2 * p1], coords[2 * p1 + 1])

            if (illegal):
                self._triangles[a] = p1
                self._triangles[b] = p0

                hbl = self._halfedges[bl]

                # edge swapped on the other side of the hull (rare); fix the halfedge reference
                if (hbl == -1):
                    e = self._hullStart
                    
                    while True:
                        if (self.hullTri[e] == bl):
                            self.hullTri[e] = a
                            break

                        e = self.hullPrev[e]
                        if (e == self._hullStart): break

                self._link(a, hbl)
                self._link(b, self._halfedges[ar])
                self._link(ar, bl)

                br = b0 + (b + 1) % 3

                # don't worry about hitting the cap: it can only happen on extremely degenerate input
                if (i < len(EDGE_STACK)):
                    EDGE_STACK[i] = br
                    i+=1

            else:
                if (i == 0): break
                i-=1
                a = EDGE_STACK[i]

        return ar

    def _link(self,a, b):
        self._halfedges[a] = b
        if (b != -1):
            self._halfedges[b] = a

    # add a new triangle given vertex indices and adjacent half-edge ids
    def _addTriangle(self,i0, i1, i2, a, b, c):
        t = self.trianglesLen

        self._triangles[t] = i0
        self._triangles[t + 1] = i1
        self._triangles[t + 2] = i2

        self._link(t, a)
        self._link(t + 1, b)
        self._link(t + 2, c)

        self.trianglesLen += 3

        return t

# monotonically increases with real angle, but doesn't need expensive trigonometry
def pseudoAngle(dx, dy):
    p = dx / (abs(dx) + abs(dy))

    if (dy > 0):
        return (3 - p) / 4 # [0..1]
    else:
        return (1 + p) / 4 # [0..1]

def dist(ax, ay, bx, by):
    dx = ax - bx
    dy = ay - by
    return dx * dx + dy * dy

# return 2d orientation sign if we're confident in it through J. Shewchuk's error bound check
def orientIfSure(px, py, rx, ry, qx, qy):
    l = (ry - py) * (qx - px)
    r = (rx - px) * (qy - py)

    if (abs(l - r) >= 3.3306690738754716e-16 * abs(l + r)):
        return l - r
    else:
        return 0

# a more robust orientation test that's stable in a given triangle (to fix robustness issues)
def orient(rx, ry, qx, qy, px, py):
    return (orientIfSure(px, py, rx, ry, qx, qy) or\
        orientIfSure(rx, ry, qx, qy, px, py) or\
        orientIfSure(qx, qy, px, py, rx, ry)) < 0

def inCircle(ax, ay, bx, by, cx, cy, px, py):
    dx = ax - px
    dy = ay - py
    ex = bx - px
    ey = by - py
    fx = cx - px
    fy = cy - py

    ap = dx * dx + dy * dy
    bp = ex * ex + ey * ey
    cp = fx * fx + fy * fy

    return dx * (ey * cp - bp * fy) -\
           dy * (ex * cp - bp * fx) +\
           ap * (ex * fy - ey * fx) < 0

def circumradius(ax, ay, bx, by, cx, cy):
    dx = bx - ax
    dy = by - ay
    ex = cx - ax
    ey = cy - ay

    bl = dx * dx + dy * dy
    cl = ex * ex + ey * ey
    d = 0.5/(dx * ey - dy * ex)

    x = (ey * bl - dy * cl) * d
    y = (dx * cl - ex * bl) * d

    return x*x + y*y

def circumcenter(ax, ay, bx, by, cx, cy):
    dx = bx - ax
    dy = by - ay
    ex = cx - ax
    ey = cy - ay

    bl = dx * dx + dy * dy
    cl = ex * ex + ey * ey
    d = 0.5/(dx * ey - dy * ex)

    x = ax + (ey * bl - dy * cl) * d
    y = ay + (dx * cl - ex * bl) * d

    return x, y

def quicksort(ids, dists, left, right):
    if (right - left <= 20):
        for i in range(left + 1,right+1):
            temp = ids[i]
            tempDist = dists[temp]
            j = i-1
            while (j >= left and dists[ids[j]] > tempDist):
                ids[j + 1] = ids[j]
                j-=1
            ids[j + 1] = temp;

    else:
        median = (left + right) >> 1
        i = left + 1
        j = right
        swap(ids, median, i)

        if (dists[ids[left]] > dists[ids[right]]):
            swap(ids, left, right)

        if (dists[ids[i]] > dists[ids[right]]):
            swap(ids, i, right)

        if (dists[ids[left]] > dists[ids[i]]):
            swap(ids, left, i)

        temp = ids[i]
        tempDist = dists[temp]

        while True:
            while True:
                i+=1
                if (dists[ids[i]] >= tempDist): break

            while True:
                j-=1
                if (dists[ids[j]] <= tempDist): break

            if (j < i): break
            swap(ids, i, j);

        ids[left + 1] = ids[j];
        ids[j] = temp;

        if (right - i + 1 >= j - left):
            quicksort(ids, dists, i, right)
            quicksort(ids, dists, left, j - 1)

        else:
            quicksort(ids, dists, left, j - 1)
            quicksort(ids, dists, i, right)

def swap(arr, i, j):
    tmp = arr[i]
    arr[i] = arr[j]
    arr[j] = tmp


from cmath import exp
from math import pi
from .dft import FFT,NthRootOfUnity
# from .idft import dft,NthRootOfUnity

class Polynomial:
    def __init__(self, listOfInt=None):
        if listOfInt is None:
            listOfInt = []
        self.listOfInt = listOfInt
    def __repr__(self):
        rep = ""
        for i in range(len(self.listOfInt)):
            rep += str(self.listOfInt[i]) + str('x^') + str(i) + str(' + ');
        rep = str.removesuffix(rep,'+ ')
        return rep
    def __add__(self, ply2):
        if len(self.listOfInt) != len(ply2.listOfInt):
            raise IndexError
        else:
            return Polynomial([self.listOfInt[i]+ply2.listOfInt[i] for i in range(len(self.listOfInt))])
    def __sub__(self, ply2):
        if len(self.listOfInt) != len(ply2.listOfInt):
            raise IndexError
        else:
            return Polynomial([self.listOfInt[i]-ply2.listOfInt[i] for i in range(len(self.listOfInt))])
    def __mul__(self, ply2):
        n = 1<<(len(self.listOfInt)+len(ply2.listOfInt)-2).bit_length()
        o = NthRootOfUnity(n)
        AT = FFT(self.listOfInt, o)
        BT = FFT(ply2.listOfInt, o)
        C = [AT[i]*BT[i] for i in range(n)]
        # nm = (len(self.listOfInt)+len(ply2.listOfInt)-1)
        D = [round((a/n).real) for a in FFT(C, o ** -1)]
        while len(D) > 0 and D[-1] == 0:
            del D[-1]
        return Polynomial(D)
# //==============================================================
# l = [1,2,3,4]
# p = Polynomial(l)
# l2 = [11,22,33,44]
# p2 = Polynomial(l2)
# print(p)
# print(p2)
# print(p+p2)
# print(p-p2)
# print(p*p2)


'''
    Constants
'''
from sys import float_info

Epsilon = float_info.epsilon
Half_Pi = 1.5707963267948966
Two_Pi = 6.283185307179586
Sqrt_3 = 1.7320508075688772


def nearly_eq(valA, valB, maxf=None, minf=None, epsilon=None):
    '''
    implementation based on:

    http://floating-point-gui.de/errors/comparison/
    '''

    if valA == valB:
        return True

    if maxf is None:
        maxf = float_info.max

    if minf is None:
        minf = float_info.min

    if epsilon is None:
        epsilon = float_info.epsilon

    absA = abs(valA)
    absB = abs(valB)
    delta = abs(valA - valB)

    if (valA == 0) or (valB == 0) or (delta < minf):
        return delta < (epsilon * minf)

    return (delta / min(absA + absB, maxf)) < (epsilon * 2)


def nearly_zero(value, minf=None, epsilon=None):

    if value == 0:
        return True

    absV = abs(value)

    if minf is None:
        minf = float_info.min

    if epsilon is None:
        epsilon = float_info.epsilon

    return absV < (epsilon * minf)



''' geometry exceptions

'''


class ZeroSlope(Exception):
    '''
    '''
    pass


class InfiniteSlope(Exception):
    '''
    '''
    pass


class CollinearPoints(Exception):
    '''
    '''
    pass


class CollinearLines(CollinearPoints):
    '''
    '''
    pass


class ParallelLines(Exception):
    '''
    '''
    pass


class InfiniteLength(Exception):
    '''
    '''
    pass


class ExceededEpsilonError(Exception):
    '''
    '''

    def __init(self, x, y, epsilon):
        self.x, self.y, self.epsilon = x, y, epsilon

    def __str__(self):
        fmt = 'x={} - y={} {} > epsilon {}'
        return fmt.format(self.x, self.y,
                          self.x - self.y,
                          self.epsilon)



from __future__ import division

from functools import reduce
import math
from numbers import Real
from .point import Point
from .line import Line

class Circle:
    def __init__(self, r):
        self.r = r
    def area(self) :
        return (math.pi*self.r*self.r)
    def findLatticePoints(self):
        r = self.r
        if (r <= 0):
            return 0  
        result = 4 
        for x in range(1, r):
            ySquare = r*r - x*x 
            y = int(math.sqrt(ySquare)) 
            if (y*y == ySquare):
                result += 4 
        return result 
    def isInside(self,point):
        x = point.x
        y = point.y
        r = self.r
        if x*x+y*y-r*r > 0 :
            return False
        if x*x+y*y-r*r == 0 :
            return False
        if x*x+y*y-r*r < 0 :
            return True
    def isOn(self,point):
        x = point.x
        y = point.y
        r = self.r
        if x*x+y*y-r*r == 0 :
            return True
        else :
            return False
    def getTangent(self,point):
        x = point.x
        y = point.y
        r = self.r
        if self.isOn(point):
            return Line(x,y,r*r)

# ==================================TEST=================================

# c = Circle(2)
# print(c.area(), c.findLatticePoints())
# p = Point(0,0)
# # print(p)
# p1 = Point(2,0)
# print(c.isInside(p))
# print(c.isOn(p1))
# print(c.getTangent(p1))


# from  .exceptions  import * 
# from  .point import *
# from  .line  import * 
# from  .ellipse  import * 
# from  .polygon  import * 
# from  .hyperbola  import * 
# from  .triangle  import * 
# from  .rectangle  import * 
# from  .graph  import * 
# from  .delaunay  import * 
# from  .dft import *
# from  .idft import *
# from  .polynomial import *

# p1 = Point(1,1)
# p2 = Point(2,7)

# print(Point.euclideanDistance(p1,p2))


# A =[1,2,3,4]
# print(dft(A))
# print(idft(A))
# print(fpm(A,A))

# from Geom8ry.point import Point 
# from Geom8ry.point import PolarPoint
# # from Geom8ry import * 
# p1 = Point(1,1)
# print(p1)
# pp1 = PolarPoint(2,30)
# p2 = pp1.toCratesian()
# print(p2)
# p3 = Point(2,2)
# # Point()
# print(Point.orientation(p1,p2,p3))
# print(Point.findQuadrant(p2))
# print(Point.euclideanDistance(p1,p2))
# print(Point.manhattanDistance(p1,p2))
# l = [p1,p2,p3]
# print(Point.centroid(l))
# print(Point.convexHull(l))
# # print(Point.closestPairOfPoints(l))
# print(Point.delaunayTriangulation(l))


from cmath import exp
from math import pi

# A simple class to simulate n-th root of unity
# This class is by no means complete and is implemented
# merely for FFT and FPM algorithms
class NthRootOfUnity:
    def __init__(self, n, k = 1):
        self.k = k
        self.n = n

    def __pow__(self, other):
        if type(other) is int:
            n = NthRootOfUnity(self.n, self.k * other)
            return n

    def __eq__(self, other):
        if other == 1:
            return abs(self.n) == abs(self.k)

    def __mul__(self, other):
        return exp(2*1j*pi*self.k/self.n)*other

    def __repr__(self):
        return str(self.n) + "-th root of unity to the " + str(self.k)

    @property
    def th(self):
        return abs(self.n // self.k)


# The Fast Fourier Transform Algorithm
#
# Input: A, An array of integers of size n representing a polynomial
#        omega, a root of unity
# Output: [A(omega), A(omega^2), ..., A(omega^(n-1))]
# Complexity: O(n logn)
def FFT(A, omega):
    if omega == 1:
        return [sum(A)]
    o2 = omega**2
    C1 = FFT(A[0::2], o2)
    C2 = FFT(A[1::2], o2)
    C3 = [None]*omega.th
    for i in range(omega.th//2):
        C3[i] = C1[i] + omega**i * C2[i]
        C3[i+omega.th//2] = C1[i] - omega**i * C2[i]
    return C3

# The Fast Polynomial Multiplication Algorithm
#
# Input: A,B, two arrays of integers representing polynomials
#        their length is in O(n)
# Output: Coefficient representation of AB
# Complexity: O(n logn)
def idft(A):
    n = 1<<(len(A)-1).bit_length()
    o = NthRootOfUnity(n)
    # return [round((a/n).real) for a in FFT(A, o ** -1)]
    # return [((a/n).real) for a in FFT(A, o ** -1)]
    return [((a/n)) for a in FFT(A, o ** -1)]

# =======================================================

# l = [1,2,3,4]

# print(idft(l))


# from  exceptions  import * 
# from  point import *
# from  line  import * 
# from  ellipse  import * 
# from  polygon  import * 
# from  hyperbola  import * 
# from  triangle  import * 
# from  rectangle  import * 
# from  graph  import * 
# from  delaunay  import * 



class Matrix:
    def __init__(self, mat):
        l_size = len(mat[0])
        for line in mat:
            if l_size != len(line):
                raise ValueError('invalid matrix sizes')
        self._raw = mat

    @property
    def raw(self):
        return self._raw

    @property
    def trace(self):
        if self.size[0] == self.size[1]:
            return sum([ self[i][j] for j in range(self.size[0]) for i in range(self.size[1]) ])
        else: print('nb lines != nb columns')

    @property
    def size(self):
        return self._raw.__len__(), self._raw[0].__len__()

    def __str__(self):
        s = "\n"
        for l in self._raw:
            s +='| '
            for c in l:
                s += '{:6.2f} '.format( round(float(c), 3))
            s +=  '|\n'
        return s
    
    def __call__(self, index):
        return self.col(self, index)

    @classmethod
    def col(cls, matrix, index, raw = False):
        col = [ line[index] for line in matrix._raw ]
        if raw: return col
        else: return Vector(col)

    def transpose(self):
        return Matrix([ self.col(self, i, True) for i in range(self.size[1]) ])

    def __setitem__(self, key, item):
        if not type(item).__name__ == 'list' or len(item) != self.size[0]:
            print('invalid assignement')
        else:
            self._raw[key] = item

    def __getitem__(self, key):
        return Vector(self._raw[key], transpose = True)

    def __add__(self, other):
        if type(other).__name__ == 'Matrix' or type(other).__name__ == 'Vector':
            if self.size[0] == other.size[0] and self.size[1] == other.size[1]:
                return Matrix([ [ self._raw[i][j] + other._raw[i][j] for j in range(self.size[1])] for i in range(self.size[0])])
        else:
            try: return Matrix([ [ self._raw[i][j] + other for j in range(self.size[1])] for i in range(self.size[0])])
            except: print('cannot add')
        
    def __sub__(self, other):
        if type(other).__name__ == 'Matrix' or type(other).__name__ == 'Vector':
            if self.size[0] == other.size[0] and self.size[1] == other.size[1]:
                return Matrix([ [ self._raw[i][j] - other._raw[i][j] for j in range(self.size[1])] for i in range(self.size[0])])
        else:
            try: return Matrix([ [ self._raw[i][j] - other for j in range(self.size[1])] for i in range(self.size[0])])
            except: print('cannot substract')

    def __mul__(self, other):
        if type(other).__name__ == 'Matrix' or type(other).__name__ == 'Vector':
            if self.size[1] == other.size[0]: # nb c == nb l
                res = []
                for i in range(self.size[0]):
                    res += [[]]
                    for j in range(other.size[1]):
                        res[i] += [sum([m*n for m,n in zip(self._raw[i], self.col(other, j, True))])]
                return Matrix(res)
        else:
            try: return Matrix([ [ self[i][j] * other for j in range(self.size[1])] for i in range(self.size[0])])
            except: print('cannot substract')

    @classmethod
    def gen(cls, l, c, fill = 0):
        mat = [[fill for j in range(c)] for i in range(l)]
        return Matrix(mat)


class Vector(Matrix):
    def __init__(self, vect, transpose = False):
        self.transposed = transpose
        super().__init__([vect] if transpose else [ [elem] for elem in vect ] )

    @property
    def raw(self):
        if self.transposed:
            return self._raw[0]
        else:
            return self.col(self, 0, True)

    @property
    def gravity(self):
        return sum(self.raw) / len(self.raw)

    def __setitem__(self, key, item):
        if self.transposed:
            self._raw[0][key] = item
        else:
            self._raw[key][0] = item

    def __getitem__(self, key):
        if self.transposed:
            if type(self._raw[0][key]).__name__ == 'list':
                return Vector(self._raw[0][key])
            else:
                return self._raw[0][key]
        else:
            if type(self._raw[key][0]).__name__ == 'list':
                return Vector(self._raw[key][0])
            else:
                return self._raw[key][0]
    
    @classmethod
    def gen(cls, l, fill = 0):
        mat = super().gen(l, 1, fill)
        return mat(0)

# ==================================================================

# m = [[1,2,3],[4,5,6],[7,8,9]]

# mt = Matrix(m)
# print(mt)
# print(mt.transpose())
# print(mt.trace)
# print(mt.size)

# mt2 =  mt.transpose()
# mt3 = Matrix.gen(3,3,9)
# print(mt3)
# print(mt2 + mt,mt2 - mt, mt2 * mt)


import math
from .point import Point
from fractions import *

# y=mx+c
class SlopeIntercept:
    def __init__(self, m, c):
        self.m = m
        self.c = c
# y-y1/x-x1 = (y2-y1)/(x2-x1)
class TwoPoint:
    def __init__(self, x1, x2, y1, y2):
        self.x1 = x1
        self.x2 = x2
        self.y1 = y1
        self.y2 = y2
# x/a + y/b = 1
class TwoIntercept:
    def __init__(self, a, b):
        self.a = a
        self.b = b
class SlopePoint():
    def __init__(self, m ,Point):
        self.m = m
        self.Point = Point
        

# ax + by = c
# Standard Lines
class Line:
    def __init__(self, a, b, c):
        self.a = a
        self.b = b
        self.c = c
    def __str__(self):
        return '('+str(self.a) +'*X + ' + str(self.b) +'*Y = '+ str(self.c)+')'
    class Transform:
        def __init__(self):
            pass
        @classmethod
        def SlopeInterceptToStandard(cls, slopeLine):
            res = Line(-slopeLine.m,1,slopeLine.c)
            return res
        @classmethod
        def TwoPointToStandard(cls , pointLine):
            res = Line(None,None,None)
            slope = Fraction(pointLine.y2-pointLine.y1,pointLine.x2-pointLine.x1)
            if slope.denominator==0:
                res = Line(1,0,pointLine.x1)
                return res
            else:
                res = Line(slope,-1,slope*pointLine.x1-pointLine.y1)
                return res
        @classmethod
        def TwoInterceptToStandard(cls , interceptLine):
            res = Line(interceptLine.b,interceptLine.a,interceptLine.a*interceptLine.b)
            return res
    @classmethod
    def distanceBetweenLines(cls, stdLine1, stdLine2):
        numerator = abs(stdLine1.c-stdLine2.c)
        denominator = math.sqrt(stdLine1.a**2 + stdLine1.b**2)
        if numerator==0:
            return 0
        return numerator/denominator
    @classmethod
    def isIntersecting(cls, stdLine1, stdLine2):
        if cls.distanceBetweenLines(stdLine1,stdLine2)==0:
            return 1
        else:
            return 0
    def rotate(self, aboutPoint, thetaClockwise):
        """ (Acosθ+Bsinθ)(x−x0)+(Bcosθ−Asinθ)(y−y0)+Ax0+By0+C=0.
        Ax+By+C=0
         """
        A = self.a*math.cos(thetaClockwise) + self.b*math.sin(thetaClockwise)
        B = self.b*math.cos(thetaClockwise) - self.a*math.sin(thetaClockwise)
        C = -(self.a*math.cos(thetaClockwise) + self.b*math.sin(thetaClockwise))*aboutPoint.x
        -(self.b*math.cos(thetaClockwise) - self.a*math.sin(thetaClockwise))*aboutPoint.y
        +self.a*aboutPoint.x + self.b*aboutPoint.y - self.c
        return Line(A,B,C)

    # determinant = a1 b2 - a2 b1
    # if (determinant == 0)
    # {
    #     // Lines are parallel
    # }
    # else
    # {
    #     x = (c1b2 - c2b1)/determinant
    #     y = (a1c2 - a2c1)/determinant
    # }

    @classmethod
    def findIntersection(cls, stdLine1,stdLine2):
        a1 = stdLine1.a
        b2 = stdLine2.b
        a2 = stdLine2.a
        b1 = stdLine1.b
        c1 = stdLine1.c
        c2 = stdLine2.c
        determinant = a1 * b2 - a2 * b1
        if determinant == 0 :
            return -1
        else:
            x = (c1*b2 - c2*b1)/determinant
            y = (a1*c2 - a2*c1)/determinant
            res = Point(x,y)
            return res

#===========================================
# p = Point(1,0)
# p1 = Point(0,1)
# l1 = SlopeIntercept(-1,1)
# # l2 = TwoPoint(p,p2)
# l3 = TwoIntercept(2,2)
# # l4 = SlopePoint(-1,p)
# print(Line.Transform.SlopeInterceptToStandard(l1))
# print(Line.Transform.TwoInterceptToStandard(l3))
# # print(Line.Transform.slopeToStandard(l1))

# print(Line.distanceBetweenLines(Line.Transform.SlopeInterceptToStandard(l1),Line.Transform.TwoInterceptToStandard(l3)))
# print(Line.isIntersecting(Line.Transform.SlopeInterceptToStandard(l1),Line.Transform.TwoInterceptToStandard(l3)))
# # print(Line.Transform.TwoInterceptToStandard(l3).rotate(Point(0,0),math.pi/2))


class Point:
    def __init__(self, x, y):
        self.x = x
        self.y = y
 
def Left_index(points):
     
    '''
    Finding the left most point
    '''
    minn = 0
    for i in range(1,len(points)):
        if points[i].x < points[minn].x:
            minn = i
        elif points[i].x == points[minn].x:
            if points[i].y > points[minn].y:
                minn = i
    return minn
 
def orientation(p, q, r):
    '''
    To find orientation of ordered triplet (p, q, r).
    The function returns following values
    0 --> p, q and r are colinear
    1 --> Clockwise
    2 --> Counterclockwise
    '''
    val = (q.y - p.y) * (r.x - q.x) - \
          (q.x - p.x) * (r.y - q.y)
 
    if val == 0:
        return 0
    elif val > 0:
        return 1
    else:
        return 2
 
def CONVEXHULL(points):
    n= len(points)
    # There must be at least 3 points
    if n < 3:
        return
 
    # Find the leftmost point
    l = Left_index(points)
 
    hull = []
     
    '''
    Start from leftmost point, keep moving counterclockwise
    until reach the start point again. This loop runs O(h)
    times where h is number of points in result or output.
    '''
    p = l
    q = 0
    while(True):
         
        # Add current point to result
        hull.append(p)
 
        '''
        Search for a point 'q' such that orientation(p, q,
        x) is counterclockwise for all points 'x'. The idea
        is to keep track of last visited most counterclock-
        wise point in q. If any point 'i' is more counterclock-
        wise than q, then update q.
        '''
        q = (p + 1) % n
 
        for i in range(n):
             
            # If i is more counterclockwise
            # than current q, then update q
            if(orientation(points[p],
                           points[i], points[q]) == 2):
                q = i
 
        '''
        Now q is the most counterclockwise with respect to p
        Set p as q for next iteration, so that q is added to
        result 'hull'
        '''
        p = q
 
        # While we don't come to first point
        if(p == l):
            break
    return hull
    # Print Result
    # for each in hull:
    #     print(points[each].x, points[each].y)
 
# # Driver Code
# points = []
# points.append(Point(0, 3))
# points.append(Point(2, 2))
# points.append(Point(1, 1))
# points.append(Point(2, 1))
# points.append(Point(3, 0))
# points.append(Point(0, 0))
# points.append(Point(3, 3))
 
# print(convexHull(points))



from __future__ import division

from functools import reduce
import math
from numbers import Real


class Point(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __repr__(self):
        return '{0}({1}, {2})'.format(
            self.__class__.__name__,
            self.x,
            self.y
        )

    def __sub__(self, point):
        """Return a Point instance as the displacement of two points."""
        if type(point) is Point:
            return self.substract(point)
        else:
            raise TypeError

    def __add__(self, pt):
        if isinstance(pt, Point):
            return Point(pt.x + self.x, pt.y + self.y)
        else:
            raise TypeError

    def __eq__(self, pt):
        return (self.x == pt.x and self.y == pt.y)

    def to_list(self):
        '''Returns an array of [x,y] of the end points'''
        return [self.x, self.y]

    def substract(self, pt):
        """Return a Point instance as the displacement of two points."""
        if isinstance(pt, Point):
                return Point(self.x- pt.x, self.y - pt.y)
        else:
            raise TypeError

    @classmethod
    def from_list(cls, l):
        """Return a Point instance from a given list"""
        if len(l) == 2:
            x, y = map(float, l)
            return cls(Point(x, y))
        elif len(l) == 2:
            x, y = map(float, l)
            return cls(Point(x, y))
        else:
            raise AttributeError


class Vector(Point):
    """Vector class: Representing a vector in 3D space.
    Can accept formats of:
    Cartesian coordinates in the x, y space.(Regular initialization)
    Spherical coordinates in the r, theta, phi space.(Spherical class method)
    Cylindrical coordinates in the r, theta space.(Cylindrical class method)
    """
    # def __init(self, Point):
    #     if Point is None:
    #         Point = Point
    #     self.Point = Point
    # def __init__(self, x, y):
    #     '''Vectors are created in rectangular coordniates
    #     to create a vector in spherical or cylindrical
    #     see the class methods
    #     '''
    #     super(Vector, self).__init__(x, y)

    def __init__(self, Point):
        '''Vectors are created in rectangular coordniates
        to create a vector in spherical or cylindrical
        see the class methods
        '''
        super(Vector, self).__init__(Point.x,Point.y)
    def __add__(self, vec):
        """Add two vectors together"""
        if(type(vec) == type(self)):
            return Vector(Point(self.x + vec.x, self.y + vec.y))
        elif isinstance(vec, Real):
            return self.add(vec)
        else:
            raise TypeError

    def __sub__(self, vec):
        """Subtract two vectors"""
        if(type(vec) == type(self)):
            return Vector(Point(self.x - vec.x, self.y - vec.y))
        elif isinstance(vec, Real):
            return Vector(Point(self.x - vec, self.y - vec))
        else:
            raise TypeError

    def __mul__(self, anotherVector):
        """Return a Vector instance as the cross product of two vectors"""
        return self.cross(anotherVector)

    def __str__(self):
        res = '(' + str(self.x) + ' , ' + str(self.y) + ')'
        return res

    def __round__(self, n=None):
        if n is not None:
            return Vector(Point(round(self.x, n), round(self.y, n)))
        return Vector(Point(round(self.x), round(self.y)))

    def add(self, number):
        """Return a Vector as the product of the vector and a real number."""
        return self.from_list([x + number for x in self.to_list()])

    def multiply(self, number):
        """Return a Vector as the product of the vector and a real number."""
        return self.from_list([x * number for x in self.to_list()])

    def magnitude(self):
        """Return magnitude of the vector."""
        return math.sqrt(
            reduce(lambda x, y: x + y, [x ** 2 for x in self.to_list()])
        )

    # def sum(self, vector):
    #     """Return a Vector instance as the vector sum of two vectors."""
    #     return self.from_list(
    #         [self.to_list()[i] + vector.to_list()[i] for i in range(2)]
    #     )

    # def subtract(self, vector):
    #     """Return a Vector instance as the vector difference of two vectors."""
    #     return self.__sub__(vector)

    def dot(self, vector, theta=None):
        """Return the dot product of two vectors.
        If theta is given then the dot product is computed as
        v1*v1 = |v1||v2|cos(theta). Argument theta
        is measured in degrees.
        """
        if theta is not None:
            return (self.magnitude() * vector.magnitude() *
                    math.degrees(math.cos(theta)))
        return (reduce(lambda x, y: x + y,
                [x * vector.to_list()[i] for i, x in enumerate(self.to_list())]))

    def cross(self, vector):
        """Return a Vector instance as the cross product of two vectors"""
        return (self.x * vector.y - self.y * vector.x)

    def unit(self):
        """Return a Vector instance of the unit vector"""
        return Vector(Point(
            (self.x / self.magnitude()),
            (self.y / self.magnitude())
        ))

    def angle(self, vector):
        """Return the angle between two vectors in degrees."""
        return math.degrees(
            math.acos(
                self.dot(vector) /
                (self.magnitude() * vector.magnitude())
            )
        )

    def parallel(self, vector):
        """Return True if vectors are parallel to each other."""
        if self.cross(vector) == 0:
            return True
        return False

    def perpendicular(self, vector):
        """Return True if vectors are perpendicular to each other."""
        if self.dot(vector) == 0:
            return True
        return False

    def non_parallel(self, vector):
        """Return True if vectors are non-parallel.
        Non-parallel vectors are vectors which are neither parallel
        nor perpendicular to each other.
        """
        if (self.parallel(vector) is not True and
                self.perpendicular(vector) is not True):
            return True
        return False

    def rotate(self, angle, axis=(0, 0, 1)):
        """Returns the rotated vector. Assumes angle is in radians"""
        if not all(isinstance(a, int) for a in axis):
            raise ValueError
        x, y = self.x, self.y

        # Z axis rotation
        if(axis[2]):
            x = (self.x * math.cos(angle) - self.y * math.sin(angle))
            y = (self.x * math.sin(angle) + self.y * math.cos(angle))

        # Y axis rotation
        if(axis[1]):
            x = self.x * math.cos(angle) 

        # X axis rotation
        if(axis[0]):
            y = self.y * math.cos(angle)

        return Vector(Point(x, y))

    def to_points(self):
        '''Returns an array of [x,y] of the end points'''
        return [self.x, self.y]


# =======================================================

# p = Point(1,1)
# v = Vector(p)
# p1 = Point(1,0)
# v1 = Vector(p1)
# print(v1+v, (v1-v), v1*v)
# print(v1.add(2), v1.magnitude(), v1.multiply(2))
# print(v1.dot(v), v1.cross(v))
# print(v1.unit(), v1.angle(v))
# print(v1.parallel(v), v1.perpendicular(v))
# print(v1.non_parallel(v))
# # print(v1.rotate(math.pi/4,(0,1,0)))
# print(v1.to_points())


import math
from fractions import *
from .point import Point
class RegularPolygon :
    def __init__(self, noOfEdges, lengthOfEdges):
        self.E = noOfEdges
        self.L = lengthOfEdges
    def area(self):
        if self.E < 3:
            raise ValueError()
            return
        else :
            numerator = self.E*self.L
            denominator = 2*math.tan(math.pi/self.E)
            return numerator/denominator
    def perimeter(self):
        if self.E < 3:
            raise ValueError()
            return
        else:
            return self.E*self.L
    def cicumradius(self):
        if self.E < 3:
            raise ValueError()
            return
        else:
            numerator = self.L
            denominator = 2*math.sin(math.pi/self.E)
            return numerator/denominator
    def inradius(self):
        if self.E < 3:
            raise ValueError()
            return
        else:
            numerator = self.L
            denominator = 2*math.tan(math.pi/self.E)
            return numerator/denominator
    def internalangles(self):
        if self.E < 3:
            raise ValueError()
            return
        else:
            return 180*(self.E-2)/self.E
# ================================================

# rp = RegularPolygon(4,2)
# print(rp.area(), rp.perimeter(), rp.cicumradius(), rp.inradius(), rp.internalangles())


'''setup for Geom8ry
'''

from setuptools import setup, find_packages
from codecs import open
from os import path


# with open("README.md", "r", encoding="utf-8") as fh:
#     long_description = fh.read()

long_description = '''
# Geom8ry - Python Geom8ry Package

## Development Stage : 2 - Pre-Alpha 
## Warning : Unstable
Version: 0.0.1

Geom8ry is a python package for performing computational
geometry. Geom8ry can handle various 2D - geometrical constructs like Points, Lines, Line Segments,  Polygons, Circles,  Fourier Transforms, Matrices, Graphs, Triangulations, and  Polynomials, etc.

It's also a personal exercise in learning how to provide a high
quality python package:

- support of powerful tools like Delaunay Triangulations, FFTs, Inverse FFTs, Convex Hulls, and Fast Polynomial Multiplications
- appropriate use of exceptions and exception handling
- designing of useful base classes and methods
- high-quality documentation
- efficient implementation of algorithms and data structures
'''

version = "0.0.1"

# download_url = 'https://github.com/raje-ev/Geom8ry/archive/{}.tar.gz'

setup(name='Geom8ry',
      version=version,
      description='Geom8ry the python way.',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/raje-ev/Geom8ry',
      # download_url=download_url.format(version),
      author="Rajeev Kumar",
      author_email="rajeevkumar.nitp@gmail.com",
      license='MIT',
      project_urls={
        "Bug Tracker": "https://github.com/raje-ev/Geom8ry/issues",
      },
      classifiers=['Development Status :: 2 - Pre-Alpha',
                   'Intended Audience :: Education',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   'Topic :: Scientific/Engineering :: Mathematics',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.6',
                   "Operating System :: OS Independent"],
      keywords='Geom8ry point circle line line segment triangle rectangle graph fft dft triangulation',
      package_dir={"": "src"},
      packages=find_packages(where="src"),
      python_requires=">=3.6",
      test_suite='Geom8ry.tests',
      install_requires=[],
      extras_require={},
      package_data={},
      data_files=[],
      )



import math
from fractions import *
from .point import Point


class Polygon :
    def __init__(self, lop=None):
        if lop is None:
         lop = []
        self.lop = lop
    def __repr__(self):
        rep = ''
        for i in range(len(self.lop)):
            rep += str('(')+(str((self.lop[i]).x)  + str(',') + str(self.lop[i].y))+') '
        return rep 
    def area(self):
        """ #Area bounded by list of Points """
        polygonArea = 0.0
        noOfPoints = len(self.lop)
        j = noOfPoints - 1
        for i in range(0,noOfPoints) :
            polygonArea += (self.lop[i].x + self.lop[j].x) * (self.lop[j].y - self.lop[i].y)
            j = i
        return abs(polygonArea / 2.0)
    def perimeter(self):
        perimeter = 0
        for i in range(0, len(self.lop)-1):
            perimeter += Point.euclideanDistance(self.lop[i],self.lop[i+1])
        perimeter += Point.euclideanDistance(self.lop[-1],self.lop[0])
        return perimeter
# ======================================================
# p1 = Point(2,0)
# p2 = Point(2,2)
# p3 = Point(0,2)
# p4 = Point(0,0)

# l= [p1,p2,p3,p4]
# p = Polygon(l)

# print(p)
# print(p.area())
# print(p.perimeter())


# import math
# from fractions import *
# from  exceptions  import * 
# from  point import *
# from  line  import * 
# from  ellipse  import * 
# from  polygon  import * 
# from  hyperbola  import * 
# from  triangle  import * 
# from  rectangle  import * 
# from  graph  import * 
# from  delaunay  import * 


class Segment:
    def __init__(self, p, q):
        self.p = p
        self.q = q
    @classmethod
    def findIntersection(cls, segment):
        return None


from __future__ import division

from functools import reduce
import math
from numbers import Real


class Hyperbola(object):
    def __init__(self, a, b):
        self.a = a
        self.b = b
        if (math.sqrt(1+(a*a)/(b*b)) < 1):
            raise AttributeError

    @classmethod
    def conjugate(cls, hyperbola):
        """Return a Conjugate of the hyperbola"""
        return Hyperbola()
            # raise AttributeError
